/**
 * Creates thunk for given promise function.
 *
 * Dispatch `actionTypes.request` action and execute function `fun`.
 * On resolved promise, `actionTypes.success` is dispatched with `payload`.
 * On error, `actionTypes.failure` is dispatched with `err`.
 *
 * @param fun - function(dispatch, getState)
 * @param actionTypes - action types object
 * @param actionTypes.request
 * @param actionTypes.success
 * @param actionTypes.failure
 * @param [actionParams={}] - params to dispatch with every action
 * @returns function
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports["default"] = createPromiseAction;

function createPromiseAction(fun, actionTypes) {
  var actionParams = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  return function (dispatch, getState) {
    dispatch(_extends({
      type: actionTypes.request
    }, actionParams));
    return fun(dispatch, getState).then(function (payload) {
      dispatch(_extends({
        type: actionTypes.success,
        payload: payload
      }, actionParams));
    })["catch"](function (err) {
      dispatch(_extends({
        type: actionTypes.failure,
        err: err
      }, actionParams));
    });
  };
}

module.exports = exports["default"];