'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.connectList = connectList;
exports.createOnSubmitHandler = createOnSubmitHandler;
exports.createOnRemoveHandler = createOnRemoveHandler;
exports.connectSingleItem = connectSingleItem;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _uuid = require('uuid');

var _uuid2 = _interopRequireDefault(_uuid);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactRedux = require('react-redux');

var _loading = require('./loading');

var _loading2 = _interopRequireDefault(_loading);

var _utils = require('../utils');

var createMapStateToProps = function createMapStateToProps(mountPoint, folder, propName) {
  if (folder === undefined) folder = '';
  return function mapStateToPropsList(state) {
    if (!state[mountPoint].hasIn(['folders', folder])) {
      return _defineProperty({}, propName, null);
    }
    var documents = state[mountPoint].get('documents');
    var ids = state[mountPoint].getIn(['folders', folder]);
    var items = ids.map(function (docId) {
      return documents.get(docId);
    });
    return _defineProperty({}, propName, items);
  };
};

exports.createMapStateToProps = createMapStateToProps;
/**
 * combines result of two mapStateToProps functions
 *
 * @param fun1
 * @param fun2
 * @returns {object} combined stateProps
 */
function combineMapStateToProps(fun1, fun2) {
  return function (state) {
    return Object.assign({}, fun1(state), fun2 ? fun2(state) : null);
  };
}

var singleObjectMapStateToProps = function singleObjectMapStateToProps(mountPoint, propName) {
  return function singleObjectMapStateToProps(state, ownProps) {
    var id = ownProps.docId;
    return _defineProperty({
      id: id
    }, propName, (0, _utils.getObjectFromState)(state, mountPoint, id));
  };
};

exports.singleObjectMapStateToProps = singleObjectMapStateToProps;

function connectList(crud, opts, mapStateToProps, mapDispatchToProps) {
  if (opts === undefined) opts = {};
  var _opts$options = opts.options;
  var options = _opts$options === undefined ? {} : _opts$options;
  var folder = opts.folder;
  var _opts$propName = opts.propName;
  var propName = _opts$propName === undefined ? "items" : _opts$propName;

  var toFolder = !folder ? JSON.stringify(options) : folder;
  var mergedMapStateToProps = combineMapStateToProps(createMapStateToProps(crud.mountPoint, toFolder, propName), mapStateToProps);

  return function (WrappedComponent) {
    var action = undefined;
    if (options.fun) {
      action = crud.actions.query(toFolder, options);
    } else {
      action = crud.actions.allDocs(toFolder, _extends({
        startkey: crud.mountPoint + '-',
        endkey: crud.mountPoint + '-￿'
      }, options));
    }
    var loadFunction = function loadFunction(c) {
      c.props.dispatch(action);
    };
    return (0, _reactRedux.connect)(mergedMapStateToProps, mapDispatchToProps)((0, _loading2['default'])(loadFunction, { propName: propName })(WrappedComponent));
  };
}

/**
 * Returns onSubmit handler that dispatch `put` action
 *
 * @param {crud} crud
 * @returns {function}
 */

function createOnSubmitHandler(crud) {
  return function onSubmit(item, data, dispatch) {
    var doc = _extends({}, item, data);
    if (!doc._id) {
      doc._id = crud.mountPoint + '-' + _uuid2['default'].v4();
    }
    dispatch(crud.actions.put(doc));
  };
}

/**
 * createOnRemoveHandler
 *
 * @param crud
 * @returns {function}
 */

function createOnRemoveHandler(crud) {
  return function onRemove(dispatch, items) {
    dispatch(crud.actions.remove(items.toObject()));
  };
}

/**
 * Decorator connects passed component to include single document as `item`.
 *
 * @param {crud} crud
 */

function connectSingleItem(crud) {
  var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var _opts$propName2 = opts.propName;
  var propName = _opts$propName2 === undefined ? 'item' : _opts$propName2;

  var mapStateToProps = singleObjectMapStateToProps(crud.mountPoint, propName);
  var loadFunction = function loadFunction(c) {
    c.props.dispatch(crud.actions.get(c.props.id));
  };

  return function (WrappedComponent) {
    return (0, _reactRedux.connect)(mapStateToProps)((0, _loading2['default'])(loadFunction, { propName: propName })(WrappedComponent));
  };
}

;