'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _slicedToArray = (function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i['return']) _i['return'](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError('Invalid attempt to destructure non-iterable instance'); } }; })();

exports['default'] = pouchdbMiddleware;

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj['default'] = obj; return newObj; } }

var _service = require('./service');

var service = _interopRequireWildcard(_service);

var POUCHDB = 'Pouchdb';

exports.POUCHDB = POUCHDB;

function pouchdbMiddleware(db) {

  return function (store) {
    return function (next) {
      return function (action) {

        var pouchAPI = action[POUCHDB];

        if (typeof pouchAPI === 'undefined') {
          return next(action);
        }

        var method = pouchAPI.method;
        var params = pouchAPI.params;
        var types = pouchAPI.types;

        if (!Array.isArray(types) || types.length !== 3) {
          throw new Error('Expected an array of three action types.');
        }
        if (!types.every(function (type) {
          return typeof type === 'string';
        })) {
          throw new Error('Expected action types to be strings.');
        }

        function actionWith(data) {
          var finalAction = Object.assign({}, action, data);
          delete finalAction[POUCHDB];
          return finalAction;
        }

        var _types = _slicedToArray(types, 3);

        var requestType = _types[0];
        var successType = _types[1];
        var failureType = _types[2];

        next(actionWith({
          type: requestType
        }));

        return service[method](db, params).then(function (payload) {
          next(actionWith({
            type: successType,
            payload: payload
          }));
        })['catch'](function (error) {
          next(actionWith({
            type: failureType,
            error: error.message || 'Something bad happened'
          }));
        });
      };
    };
  };
}